/*
 *	Copyright (c) 2005, Neil Goodman
 *	All rights reserved.
 *
 *	This software is provided "as-is", without any express or implied warranty.
 *	In no event shall the author be held liable for any damages arising from
 *	the use of this software. 
 */

//	==============================================================Include Files

#include "HID.h"

extern "C"
{
	#include "setupapi.h"
	#include "hidsdi.h"
}

//	=============================================================Implementation

//	............................................................HID Enumeration

static GUID							gs_HidGuid;
static int							gs_DeviceIndex;
static HANDLE						gs_DeviceInfo;
static SP_DEVICE_INTERFACE_DATA		gs_InterfaceData;

static void OpenFindHID() 
{
	HidD_GetHidGuid(&gs_HidGuid);

	gs_DeviceInfo  = SetupDiGetClassDevs(&gs_HidGuid, NULL, NULL, DIGCF_PRESENT | DIGCF_INTERFACEDEVICE);
	gs_DeviceIndex = 0;
}

static SP_DEVICE_INTERFACE_DATA* FindNextHID() 
{
	gs_InterfaceData.cbSize = sizeof(gs_InterfaceData);

	if (SetupDiEnumDeviceInterfaces(gs_DeviceInfo, 0, &gs_HidGuid, gs_DeviceIndex++, &gs_InterfaceData) == 0)
	{
		//	No more devices
		return (NULL);
	}

	//	Return the device interface data
	return (&gs_InterfaceData);
}

static void CloseFindHID() 
{
	SetupDiDestroyDeviceInfoList(gs_DeviceInfo);
	gs_DeviceInfo = NULL;
}

//	.................................................................HID Detail

static PSP_DEVICE_INTERFACE_DETAIL_DATA	GetHIDDetail(SP_DEVICE_INTERFACE_DATA* pInterfaceData) 
{
	//	Determine the size of the detail data
	ULONG Length = 0;
	SetupDiGetDeviceInterfaceDetail(gs_DeviceInfo, pInterfaceData, NULL, 0, &Length, NULL);

	//	Allocate a buffer for the detail data
	PSP_DEVICE_INTERFACE_DETAIL_DATA pDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA) malloc(Length);
	pDetailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

	//	Obtain the device interface's detail data
	SetupDiGetDeviceInterfaceDetail(gs_DeviceInfo, pInterfaceData, pDetailData, Length, NULL, NULL);

	return (pDetailData);
}

static void ReleaseHIDDetail(PSP_DEVICE_INTERFACE_DETAIL_DATA pDetailData) 
{
	free(pDetailData);
}

//	...........................................................................

HANDLE OpenHID(int VendorID, int ProductID) 
{
	HANDLE hHID = INVALID_HANDLE_VALUE;

	//	Iterate over every present device
	OpenFindHID();

	while (true)
	{
		SP_DEVICE_INTERFACE_DATA* pInterfaceData = FindNextHID();

		if (pInterfaceData == NULL)
		{
			break;
		}

		//	Obtain the device's details
		PSP_DEVICE_INTERFACE_DETAIL_DATA pDetailData = GetHIDDetail(pInterfaceData);

		if (pDetailData != NULL)
		{
			//	Open a handle to the device
			HANDLE hDevice = CreateFile(pDetailData->DevicePath, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);

			if (hDevice != NULL)
			{
				//	Read the device attributes
				HIDD_ATTRIBUTES Attributes;
				{
					Attributes.Size = sizeof(Attributes);
					HidD_GetAttributes(hDevice, &Attributes);
				}

				//	Is this the Sky controller device?
				if ((Attributes.VendorID  == VendorID) &&
					(Attributes.ProductID == ProductID))
				{
					//	Sky controller detected

					//	Get the device's capabilities
					HIDP_CAPS Capabilities;
					PHIDP_PREPARSED_DATA PreparsedData;
					HidD_GetPreparsedData(hDevice, &PreparsedData);
					HidP_GetCaps(PreparsedData, &Capabilities);
					HidD_FreePreparsedData(PreparsedData);

					//	Open the usable handle to the device
					hHID = CreateFile(pDetailData->DevicePath, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);

					break;
				}

				//	Release the device handle
				CloseHandle(hDevice);
			}

			//	Release the detail data
			ReleaseHIDDetail(pDetailData);
		}
	}

	//	Release the HID enumerator
	CloseFindHID();

	return (hHID);
}

//	===========================================================================
