/*
 *	Copyright (c) 2005, Neil Goodman
 *	All rights reserved.
 *
 *	This software is provided "as-is", without any express or implied warranty.
 *	In no event shall the author be held liable for any damages arising from
 *	the use of this software. 
 */

//	==============================================================Include Files

#include "SkyController.h"
#include "HID.h"

//	=============================================================Implementation

//	................................................Constructors And Destructor

SkyController::SkyController() 
{
	m_DeviceHandle = INVALID_HANDLE_VALUE;
}

SkyController::~SkyController() 
{
	CloseDevice();
}

//	..........................................................Device Management

bool SkyController::OpenDevice() 
{
	//	Check if the device is already open
	if (IsDeviceOpen())
	{
		return (true);
	}

	//	Open the device
	const int VendorID  = 0x6666;
	const int ProductID = 0xF100;

	m_DeviceHandle = OpenHID(VendorID, ProductID);

	//	Return the result
	return (IsDeviceOpen());
}

void SkyController::CloseDevice() 
{
	//	Close the device if it's open
	if (IsDeviceOpen())
	{
		CloseHandle(m_DeviceHandle);
		m_DeviceHandle = INVALID_HANDLE_VALUE;
	}
}

bool SkyController::IsDeviceOpen() 
{
	return (m_DeviceHandle != INVALID_HANDLE_VALUE);
}

//	...................................................................Commands

bool SkyController::SendCommand(EOutput Output, EBox Box, ECommand Command) 
{
	//	Check if the device is open
	if (IsDeviceOpen() == false)
	{
		return (false);
	}

	//	Send the command
	BYTE  Buffer[]     = { 0, Output, Box, Command, 122, 122, 122, 122, 122 };
	DWORD BytesWritten = 0;
	BOOL  Success      = WriteFile(m_DeviceHandle, Buffer, sizeof(Buffer), &BytesWritten, NULL);

	//	Return the result
	return (Success ? true : false);
}

//	===========================================================================
