/*
 *	Copyright (c) 2005, Neil Goodman
 *	All rights reserved.
 *
 *	This software is provided "as-is", without any express or implied warranty.
 *	In no event shall the author be held liable for any damages arising from
 *	the use of this software. 
 */

//	==============================================================Include Files

#include <windows.h>
#include <stdio.h>
#include <time.h>

#include "SkyController.h"

//	=============================================================Implementation

//	...........................................................Global Constants

#ifdef BOX_SKY
	#ifdef OUTPUT_ONE
		static const EBox		gs_Box            = eBox_Sky;
		static const EOutput	gs_Output         = eOutput_One;
		static const char*		gs_pszName        = "Sky Tuner";
		static const char*		gs_pszDescription = "BeyondTV tuner driver for a Sky digibox.";
	#else
		static const EBox		gs_Box            = eBox_Sky;
		static const EOutput	gs_Output         = eOutput_Two;
		static const char*		gs_pszName        = "Sky Tuner (2nd Output)";
		static const char*		gs_pszDescription = "BeyondTV tuner driver for a Sky digibox using output #2.";
	#endif
#else
	#ifdef OUTPUT_ONE
		static const EBox		gs_Box            = eBox_SkyPlus;
		static const EOutput	gs_Output         = eOutput_One;
		static const char*		gs_pszName        = "Sky+ Tuner";
		static const char*		gs_pszDescription = "BeyondTV tuner driver for a Sky+ digibox.";
	#else
		static const EBox		gs_Box            = eBox_SkyPlus;
		static const EOutput	gs_Output         = eOutput_Two;
		static const char*		gs_pszName        = "Sky+ Tuner (2nd Output)";
		static const char*		gs_pszDescription = "BeyondTV tuner driver for a Sky+ digibox using output #2.";
	#endif
#endif

//	.........................................................BeyondTV Interface

struct TTuner
{
	DWORD	dwVersionSpec;				//	Interface version number (currently 2)
	DWORD	dwVersion;					//	Internal version number
	char	szName[MAX_PATH];			//	Human readable name of the device
	char	szDescription[MAX_PATH];	//	Human readable description
	char	szFilename[MAX_PATH];		//	Used internally by BeyondTV, do not use or modify
};

int TunerGetProperties(TTuner* pTuner) 
{
	pTuner->dwVersionSpec = 2;
	pTuner->dwVersion     = 1;

	strcpy_s(pTuner->szName,        sizeof(pTuner->szName),        gs_pszName);
	strcpy_s(pTuner->szDescription, sizeof(pTuner->szDescription), gs_pszDescription);

	return (1);
}

BOOL SetTunerFrequency(DWORD Channel, DWORD BroadcastType, DWORD Frequency)
{
	//	Open communications with the controller
	SkyController Controller;

	if (Controller.OpenDevice())
	{
		//	Send the 'Sky' command to turn the box on and exit the interactive
		//	  services, etc.
		Controller.SendCommand(gs_Output, gs_Box, eCommand_Sky);

		//	Finally, send the three channel digits
		Controller.SendCommand(gs_Output, gs_Box, (ECommand) (eCommand_0 + ((Channel / 100) % 10)));
		Controller.SendCommand(gs_Output, gs_Box, (ECommand) (eCommand_0 + ((Channel /  10) % 10)));
		Controller.SendCommand(gs_Output, gs_Box, (ECommand) (eCommand_0 + ((Channel /   1) % 10)));
	}

	return (TRUE);
}

//	===========================================================================
